@@ -14,14 +14,14 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #ifndef __XFCONF_TESTS_COMMON_H__
 #define __XFCONF_TESTS_COMMON_H__
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #ifdef HAVE_SYS_TYPES_H
@@ -40,20 +40,24 @@
 #include <unistd.h>
 #endif
 
+#include <gio/gio.h>
 #include <glib.h>
-#include <dbus/dbus.h>
-#include <xfconf/xfconf.h>
 
-#define TEST_CHANNEL_NAME  "test-channel"
-#define WAIT_TIMEOUT       15
+#include "xfconf/xfconf.h"
 
-#define TEST_OPERATION(x) G_STMT_START{ \
-    if(!(x)) { \
-        g_critical("Test failed: " # x); \
-        xfconf_tests_end(); \
-        return 1; \
+#define TEST_CHANNEL_NAME "test-channel"
+#define WAIT_TIMEOUT 15
+
+#define TEST_OPERATION(x) \
+    G_STMT_START \
+    { \
+        if (!(x)) { \
+            g_critical("Test failed: " #x); \
+            xfconf_tests_end(); \
+            return 1; \
+        } \
     } \
-}G_STMT_END
+    G_STMT_END
 
 /* don't use static to avoid compiler warnings in tests that don't use
  * all of them */
@@ -76,49 +80,65 @@ static void xfconf_tests_end();
 static gboolean
 xfconf_tests_start(void)
 {
-    DBusConnection *dbus_conn;
-    DBusMessage *msg, *ret;
-    DBusError derror;
-    GTimeVal start, now;
+    GDBusConnection *conn;
+    GDBusMessage *msg, *ret;
+    gint64 start, now;
     GError *error = NULL;
 
     /* wait until xfconfd finishes starting */
-    dbus_error_init(&derror);
-    dbus_conn = dbus_bus_get(DBUS_BUS_SESSION, NULL);
-    if(!dbus_conn) {
-        g_critical("Failed to connect to D-Bus: %s", derror.message);
-        dbus_error_free(&derror);
+    conn = g_bus_get_sync(G_BUS_TYPE_SESSION, NULL, &error);
+    if (!conn) {
+        g_critical("Failed to connect to D-Bus: %s", error->message);
+        g_error_free(error);
         xfconf_tests_end();
         return FALSE;
     }
-    msg = dbus_message_new_method_call("org.xfce.Xfconf",
-                                      "/org/xfce/Xfconf",
-                                      "org.freedesktop.DBus.Peer",
-                                      "Ping");
-    g_get_current_time(&start);
-    while(!(ret = dbus_connection_send_with_reply_and_block(dbus_conn,
-                                                            msg, 0, NULL)))
+    msg = g_dbus_message_new_method_call(XFCONF_SERVICE_NAME_PREFIX ".XfconfTest",
+                                         XFCONF_SERVICE_PATH_PREFIX "/Xfconf",
+                                         "org.freedesktop.DBus.Peer",
+                                         "Ping");
+    start = g_get_monotonic_time();
+    while (!(ret = g_dbus_connection_send_message_with_reply_sync(conn,
+                                                                  msg,
+                                                                  G_DBUS_SEND_MESSAGE_FLAGS_NONE,
+                                                                  -1, NULL, NULL, NULL))
+           || g_dbus_message_get_message_type(ret) != G_DBUS_MESSAGE_TYPE_METHOD_RETURN)
     {
-        g_get_current_time(&now);
-        if(now.tv_sec - start.tv_sec > WAIT_TIMEOUT) {
+        if (ret != NULL) {
+            GError *dbus_error = NULL;
+            if (g_dbus_message_to_gerror(ret, &dbus_error)) {
+                g_critical("D-Bus error: %s", dbus_error->message);
+                g_error_free(dbus_error);
+            }
+            g_object_unref(ret);
+            g_object_unref(msg);
+            msg = g_dbus_message_new_method_call(XFCONF_SERVICE_NAME_PREFIX ".XfconfTest",
+                                                 XFCONF_SERVICE_PATH_PREFIX "/Xfconf",
+                                                 "org.freedesktop.DBus.Peer",
+                                                 "Ping");
+        }
+
+        now = g_get_monotonic_time();
+        if (now - start > WAIT_TIMEOUT * G_USEC_PER_SEC) {
             g_critical("xfconfd failed to start after %d seconds", WAIT_TIMEOUT);
-            dbus_message_unref(msg);
-            dbus_connection_unref(dbus_conn);
+            g_object_unref(msg);
             xfconf_tests_end();
             return FALSE;
         }
-    }
-    dbus_message_unref(msg);
-    dbus_message_unref(ret);
-    dbus_connection_unref(dbus_conn);
 
-    if(!xfconf_init(&error)) {
+        g_usleep(10000);
+    }
+
+    g_object_unref(msg);
+    g_object_unref(ret);
+
+    if (!xfconf_init(&error)) {
         g_critical("Failed to init libxfconf: %s", error->message);
         g_error_free(error);
         xfconf_tests_end();
         return FALSE;
     }
-    
+
     return TRUE;
 }
 
@@ -128,4 +148,4 @@ xfconf_tests_end(void)
     xfconf_shutdown();
 }
 
-#endif  /* __XFCONF_TESTS_COMMON_H__ */
+#endif /* __XFCONF_TESTS_COMMON_H__ */
